package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.KeyParameter;

public class CamelliaEngine implements BlockCipher {
   private static final int BLOCK_SIZE = 16;
   private static final long MASK8 = 255L;
   private static final long MASK32 = 4294967295L;
   private static final long SIGMA1 = -6872943284670656373L;
   private static final long SIGMA2 = -5297666545706961998L;
   private static final long SIGMA3 = -4112007255848680770L;
   private static final long SIGMA4 = 6124705990439038748L;
   private static final long SIGMA5 = 1217423232700263709L;
   private static final long SIGMA6 = -5740250305213840899L;
   private boolean initialised;
   private boolean _keyIs128;
   private long _kw1;
   private long _kw2;
   private long _kw3;
   private long _kw4;
   private long _k1;
   private long _k2;
   private long _k3;
   private long _k4;
   private long _k5;
   private long _k6;
   private long _k7;
   private long _k8;
   private long _k9;
   private long _k10;
   private long _k11;
   private long _k12;
   private long _k13;
   private long _k14;
   private long _k15;
   private long _k16;
   private long _k17;
   private long _k18;
   private long _k19;
   private long _k20;
   private long _k21;
   private long _k22;
   private long _k23;
   private long _k24;
   private long _ke1;
   private long _ke2;
   private long _ke3;
   private long _ke4;
   private long _ke5;
   private long _ke6;
   private final byte[] SBOX1 = new byte[]{112, -126, 44, -20, -77, 39, -64, -27, -28, -123, 87, 53, -22, 12, -82, 65, 35, -17, 107, -109, 69, 25, -91, 33, -19, 14, 79, 78, 29, 101, -110, -67, -122, -72, -81, -113, 124, -21, 31, -50, 62, 48, -36, 95, 94, -59, 11, 26, -90, -31, 57, -54, -43, 71, 93, 61, -39, 1, 90, -42, 81, 86, 108, 77, -117, 13, -102, 102, -5, -52, -80, 45, 116, 18, 43, 32, -16, -79, -124, -103, -33, 76, -53, -62, 52, 126, 118, 5, 109, -73, -87, 49, -47, 23, 4, -41, 20, 88, 58, 97, -34, 27, 17, 28, 50, 15, -100, 22, 83, 24, -14, 34, -2, 68, -49, -78, -61, -75, 122, -111, 36, 8, -24, -88, 96, -4, 105, 80, -86, -48, -96, 125, -95, -119, 98, -105, 84, 91, 30, -107, -32, -1, 100, -46, 16, -60, 0, 72, -93, -9, 117, -37, -118, 3, -26, -38, 9, 63, -35, -108, -121, 92, -125, 2, -51, 74, -112, 51, 115, 103, -10, -13, -99, 127, -65, -30, 82, -101, -40, 38, -56, 55, -58, 59, -127, -106, 111, 75, 19, -66, 99, 46, -23, 121, -89, -116, -97, 110, -68, -114, 41, -11, -7, -74, 47, -3, -76, 89, 120, -104, 6, 106, -25, 70, 113, -70, -44, 37, -85, 66, -120, -94, -115, -6, 114, 7, -71, 85, -8, -18, -84, 10, 54, 73, 42, 104, 60, 56, -15, -92, 64, 40, -45, 123, -69, -55, 67, -63, 21, -29, -83, -12, 119, -57, -128, -98};
   private final byte[] SBOX2 = new byte[256];
   private final byte[] SBOX3 = new byte[256];
   private final byte[] SBOX4 = new byte[256];

   public CamelliaEngine() {
      for(int var1 = 0; var1 != 256; ++var1) {
         this.SBOX2[var1] = this.lRot8(this.SBOX1[var1], 1);
         this.SBOX3[var1] = this.lRot8(this.SBOX1[var1], 7);
         this.SBOX4[var1] = this.SBOX1[this.lRot8((byte)var1, 1) & 255];
      }

   }

   private void setKey(boolean forEncryption, byte[] key) {
      int var3;
      long var5;
      long var7;
      long var9;
      long var18;
      if ((var3 = key.length) != 16) {
         if (var3 != 24) {
            if (var3 != 32) {
               throw new IllegalArgumentException("only a key sizes of 128/192/256 are acceptable.");
            }

            var18 = this.bytesToWord(key, 0);
            var5 = this.bytesToWord(key, 8);
            var7 = this.bytesToWord(key, 16);
            var9 = this.bytesToWord(key, 24);
            this._keyIs128 = false;
         } else {
            var18 = this.bytesToWord(key, 0);
            var5 = this.bytesToWord(key, 8);
            var7 = this.bytesToWord(key, 16);
            var9 = ~this.bytesToWord(key, 16);
            this._keyIs128 = false;
         }
      } else {
         this._keyIs128 = true;
         var18 = this.bytesToWord(key, 0);
         var5 = this.bytesToWord(key, 8);
         var7 = 0L;
         var9 = 0L;
      }

      long var11;
      long var13 = (var13 = (var11 = var18 ^ var7) ^ this.f(var11 = var5 ^ var9 ^ this.f(var11, -6872943284670656373L), -5297666545706961998L) ^ var18) ^ this.f(var11 = var11 ^ var5 ^ this.f(var13, -4112007255848680770L), 6124705990439038748L);
      if (this._keyIs128) {
         if (forEncryption) {
            this._kw1 = var18;
            this._kw2 = var5;
            this._kw3 = this.lRot128high(var13, var11, 111);
            this._kw4 = this.lRot128low(var13, var11, 111);
            this._k1 = var13;
            this._k2 = var11;
            this._k3 = this.lRot128high(var18, var5, 15);
            this._k4 = this.lRot128low(var18, var5, 15);
            this._k5 = this.lRot128high(var13, var11, 15);
            this._k6 = this.lRot128low(var13, var11, 15);
            this._k7 = this.lRot128high(var18, var5, 45);
            this._k8 = this.lRot128low(var18, var5, 45);
            this._k9 = this.lRot128high(var13, var11, 45);
            this._k10 = this.lRot128low(var18, var5, 60);
            this._k11 = this.lRot128high(var13, var11, 60);
            this._k12 = this.lRot128low(var13, var11, 60);
            this._k13 = this.lRot128high(var18, var5, 94);
            this._k14 = this.lRot128low(var18, var5, 94);
            this._k15 = this.lRot128high(var13, var11, 94);
            this._k16 = this.lRot128low(var13, var11, 94);
            this._k17 = this.lRot128high(var18, var5, 111);
            this._k18 = this.lRot128low(var18, var5, 111);
            this._ke1 = this.lRot128high(var13, var11, 30);
            this._ke2 = this.lRot128low(var13, var11, 30);
            this._ke3 = this.lRot128high(var18, var5, 77);
            this._ke4 = this.lRot128low(var18, var5, 77);
         } else {
            this._kw3 = var18;
            this._kw4 = var5;
            this._kw1 = this.lRot128high(var13, var11, 111);
            this._kw2 = this.lRot128low(var13, var11, 111);
            this._k18 = var13;
            this._k17 = var11;
            this._k16 = this.lRot128high(var18, var5, 15);
            this._k15 = this.lRot128low(var18, var5, 15);
            this._k14 = this.lRot128high(var13, var11, 15);
            this._k13 = this.lRot128low(var13, var11, 15);
            this._k12 = this.lRot128high(var18, var5, 45);
            this._k11 = this.lRot128low(var18, var5, 45);
            this._k10 = this.lRot128high(var13, var11, 45);
            this._k9 = this.lRot128low(var18, var5, 60);
            this._k8 = this.lRot128high(var13, var11, 60);
            this._k7 = this.lRot128low(var13, var11, 60);
            this._k6 = this.lRot128high(var18, var5, 94);
            this._k5 = this.lRot128low(var18, var5, 94);
            this._k4 = this.lRot128high(var13, var11, 94);
            this._k3 = this.lRot128low(var13, var11, 94);
            this._k2 = this.lRot128high(var18, var5, 111);
            this._k1 = this.lRot128low(var18, var5, 111);
            this._ke4 = this.lRot128high(var13, var11, 30);
            this._ke3 = this.lRot128low(var13, var11, 30);
            this._ke2 = this.lRot128high(var18, var5, 77);
            this._ke1 = this.lRot128low(var18, var5, 77);
         }
      } else {
         boolean var10000 = forEncryption;
         long forEncryption1;
         long var15 = (forEncryption1 = var13 ^ var7) ^ this.f(forEncryption1 = var11 ^ var9 ^ this.f(forEncryption1, 1217423232700263709L), -5740250305213840899L);
         if (var10000) {
            this._kw1 = var18;
            this._kw2 = var5;
            this._k1 = var15;
            this._k2 = forEncryption1;
            this._k3 = this.lRot128high(var7, var9, 15);
            this._k4 = this.lRot128low(var7, var9, 15);
            this._k5 = this.lRot128high(var13, var11, 15);
            this._k6 = this.lRot128low(var13, var11, 15);
            this._ke1 = this.lRot128high(var7, var9, 30);
            this._ke2 = this.lRot128low(var7, var9, 30);
            this._k7 = this.lRot128high(var15, forEncryption1, 30);
            this._k8 = this.lRot128low(var15, forEncryption1, 30);
            this._k9 = this.lRot128high(var18, var5, 45);
            this._k10 = this.lRot128low(var18, var5, 45);
            this._k11 = this.lRot128high(var13, var11, 45);
            this._k12 = this.lRot128low(var13, var11, 45);
            this._ke3 = this.lRot128high(var18, var5, 60);
            this._ke4 = this.lRot128low(var18, var5, 60);
            this._k13 = this.lRot128high(var7, var9, 60);
            this._k14 = this.lRot128low(var7, var9, 60);
            this._k15 = this.lRot128high(var15, forEncryption1, 60);
            this._k16 = this.lRot128low(var15, forEncryption1, 60);
            this._k17 = this.lRot128high(var18, var5, 77);
            this._k18 = this.lRot128low(var18, var5, 77);
            this._ke5 = this.lRot128high(var13, var11, 77);
            this._ke6 = this.lRot128low(var13, var11, 77);
            this._k19 = this.lRot128high(var7, var9, 94);
            this._k20 = this.lRot128low(var7, var9, 94);
            this._k21 = this.lRot128high(var13, var11, 94);
            this._k22 = this.lRot128low(var13, var11, 94);
            this._k23 = this.lRot128high(var18, var5, 111);
            this._k24 = this.lRot128low(var18, var5, 111);
            this._kw3 = this.lRot128high(var15, forEncryption1, 111);
            this._kw4 = this.lRot128low(var15, forEncryption1, 111);
         } else {
            this._kw3 = var18;
            this._kw4 = var5;
            this._kw1 = this.lRot128high(var15, forEncryption1, 111);
            this._kw2 = this.lRot128low(var15, forEncryption1, 111);
            this._k24 = var15;
            this._k23 = forEncryption1;
            this._k22 = this.lRot128high(var7, var9, 15);
            this._k21 = this.lRot128low(var7, var9, 15);
            this._k20 = this.lRot128high(var13, var11, 15);
            this._k19 = this.lRot128low(var13, var11, 15);
            this._k18 = this.lRot128high(var15, forEncryption1, 30);
            this._k17 = this.lRot128low(var15, forEncryption1, 30);
            this._k16 = this.lRot128high(var18, var5, 45);
            this._k15 = this.lRot128low(var18, var5, 45);
            this._k14 = this.lRot128high(var13, var11, 45);
            this._k13 = this.lRot128low(var13, var11, 45);
            this._k12 = this.lRot128high(var7, var9, 60);
            this._k11 = this.lRot128low(var7, var9, 60);
            this._k10 = this.lRot128high(var15, forEncryption1, 60);
            this._k9 = this.lRot128low(var15, forEncryption1, 60);
            this._k8 = this.lRot128high(var18, var5, 77);
            this._k7 = this.lRot128low(var18, var5, 77);
            this._k6 = this.lRot128high(var7, var9, 94);
            this._k5 = this.lRot128low(var7, var9, 94);
            this._k4 = this.lRot128high(var13, var11, 94);
            this._k3 = this.lRot128low(var13, var11, 94);
            this._k2 = this.lRot128high(var18, var5, 111);
            this._k1 = this.lRot128low(var18, var5, 111);
            this._ke6 = this.lRot128high(var7, var9, 30);
            this._ke5 = this.lRot128low(var7, var9, 30);
            this._ke4 = this.lRot128high(var18, var5, 60);
            this._ke3 = this.lRot128low(var18, var5, 60);
            this._ke2 = this.lRot128high(var13, var11, 77);
            this._ke1 = this.lRot128low(var13, var11, 77);
         }
      }

   }

   private byte lRot8(byte value, int rotation) {
      return (byte)(value << rotation | (value & 255) >>> 8 - rotation);
   }

   private int lRot32(int value, int rotation) {
      return value << rotation | value >>> -rotation;
   }

   private long lRot128high(long a, long b, int rotation) {
      if (rotation < 64) {
         b = a << rotation | b >>> -rotation;
      } else if (rotation != 64) {
         int var6;
         b = b << (var6 = rotation - 64) | a >>> -var6;
      }

      return b;
   }

   private long lRot128low(long a, long b, int rotation) {
      if (rotation < 64) {
         a = b << rotation | a >>> -rotation;
      } else if (rotation != 64) {
         int var6;
         a = a << (var6 = rotation - 64) | b >>> -var6;
      }

      return a;
   }

   private long fl(long in, long ke) {
      int var5;
      int var10000 = var5 = (int)(in >> 32);
      int var10001 = (int)in;
      CamelliaEngine var10002 = this;
      int var6 = (int)(ke >> 32);
      int in1 = (int)ke;
      return (long)(var10000 ^ ((var6 = var10001 ^ var10002.lRot32(var5 & var6, 1)) | in1)) << 32 | (long)var6 & 4294967295L;
   }

   private long flInv(long in, long ke) {
      long var10000 = in;
      int in1 = (int)(in >> 32);
      int var2;
      int var7 = var2 = (int)var10000;
      CamelliaEngine var10001 = this;
      int var5 = (int)(ke >> 32);
      var5 = var7 ^ var10001.lRot32((in1 ^= var2 | (int)ke) & var5, 1);
      return (long)in1 << 32 | (long)var5 & 4294967295L;
   }

   private long f(long in, long ke) {
      long var10001 = in ^ ke;
      int in1 = (int)((in ^ ke) >> 32);
      int var2 = (int)var10001;
      int var4;
      byte[] ke1;
      byte var10000 = (byte)(var4 = (ke1 = this.SBOX1)[in1 >> 24 & 255]);
      byte[] var5;
      int var6 = (var5 = this.SBOX2)[in1 >> 16 & 255];
      byte[] var7;
      int var8;
      byte var20 = (byte)(var8 = (var7 = this.SBOX3)[in1 >> 8 & 255]);
      byte var10002 = (byte)var6;
      int var10003 = var8;
      int var10004 = var4;
      byte var10006 = (byte)var4;
      byte[] var10008 = ke1;
      int var10009 = var2;
      byte[] var11;
      byte[] var10010 = var11 = this.SBOX4;
      int var10011 = var2;
      int var12 = var11[in1 & 255];
      byte in2 = var5[var2 >> 24 & 255];
      byte var15 = var7[var2 >> 16 & 255];
      byte ke2 = var10010[var10011 >> 8 & 255];
      var4 = var10008[var10009 & 255];
      int var18 = var10006 ^ var8 ^ var12 ^ var15 ^ ke2 ^ var4;
      var10004 = var6 ^= var10004;
      int var19 = var6 ^ var12 ^ in2 ^ ke2 ^ var4;
      var8 = var10004 ^ var8 ^ in2 ^ var15 ^ var4;
      int var9 = var10002 ^ var10003;
      var10003 = var9;
      int var10005 = var6;
      var6 = var9 ^ var12 ^ in2 ^ var15 ^ ke2;
      var9 = var10005 ^ var15 ^ ke2 ^ var4;
      int var10 = var10003 ^ in2 ^ ke2 ^ var4;
      var4 ^= var20 ^ var12 ^ in2 ^ var15;
      var12 = var10000 ^ var12 ^ in2 ^ var15 ^ ke2;
      return (long)var18 << 56 | ((long)var19 & 255L) << 48 | ((long)var8 & 255L) << 40 | ((long)var6 & 255L) << 32 | ((long)var9 & 255L) << 24 | ((long)var10 & 255L) << 16 | ((long)var4 & 255L) << 8 | (long)var12 & 255L;
   }

   private long bytesToWord(byte[] src, int srcOff) {
      long var3 = 0L;

      for(int var5 = 0; var5 < 8; ++var5) {
         var3 = (var3 << 8) + (long)(src[var5 + srcOff] & 255);
      }

      return var3;
   }

   private void wordToBytes(long word, byte[] dst, int dstOff) {
      for(int var5 = 0; var5 < 8; ++var5) {
         dst[7 - var5 + dstOff] = (byte)((int)word);
         word >>>= 8;
      }

   }

   private int processBlock128(byte[] in, int inOff, byte[] out, int outOff) {
      CamelliaEngine var10000 = this;
      CamelliaEngine var10001 = this;
      long var10006 = this.bytesToWord(in, inOff);
      long in1 = this.bytesToWord(in, inOff + 8);
      long var5;
      var10006 = var5 = var10006 ^ this._kw1;
      in1 = in1 ^ this._kw2 ^ this.f(var5, this._k1);
      var5 = this._k2;
      var10006 = var5 = var10006 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k3);
      var5 = this._k4;
      var10006 = var5 = var10006 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k5);
      var5 = this._k6;
      var5 = var10006 ^ this.f(in1, var5);
      long var7 = this._ke1;
      long var10004 = var5 = this.fl(var5, var7);
      in1 = this.flInv(in1, this._ke2) ^ this.f(var5, this._k7);
      var5 = this._k8;
      var10004 = var5 = var10004 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k9);
      var5 = this._k10;
      var10004 = var5 = var10004 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k11);
      var5 = this._k12;
      var5 = var10004 ^ this.f(in1, var5);
      var7 = this._ke3;
      long var10002 = var5 = this.fl(var5, var7);
      in1 = this.flInv(in1, this._ke4) ^ this.f(var5, this._k13);
      var5 = this._k14;
      var10002 = var5 = var10002 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k15);
      var5 = this._k16;
      var10002 = var5 = var10002 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k17);
      var5 = this._k18;
      var10002 ^= this.f(in1, var5);
      CamelliaEngine var10003 = this;
      long var9 = in1 ^ this._kw3;
      var5 = var10002 ^ var10003._kw4;
      var10001.wordToBytes(var9, out, outOff);
      var10000.wordToBytes(var5, out, outOff + 8);
      return 16;
   }

   private int processBlock192or256(byte[] in, int inOff, byte[] out, int outOff) {
      CamelliaEngine var10000 = this;
      CamelliaEngine var10001 = this;
      long var10008 = this.bytesToWord(in, inOff);
      long in1 = this.bytesToWord(in, inOff + 8);
      long var5;
      var10008 = var5 = var10008 ^ this._kw1;
      in1 = in1 ^ this._kw2 ^ this.f(var5, this._k1);
      var5 = this._k2;
      var10008 = var5 = var10008 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k3);
      var5 = this._k4;
      var10008 = var5 = var10008 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k5);
      var5 = this._k6;
      var5 = var10008 ^ this.f(in1, var5);
      long var7 = this._ke1;
      long var10006 = var5 = this.fl(var5, var7);
      in1 = this.flInv(in1, this._ke2) ^ this.f(var5, this._k7);
      var5 = this._k8;
      var10006 = var5 = var10006 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k9);
      var5 = this._k10;
      var10006 = var5 = var10006 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k11);
      var5 = this._k12;
      var5 = var10006 ^ this.f(in1, var5);
      var7 = this._ke3;
      long var10004 = var5 = this.fl(var5, var7);
      in1 = this.flInv(in1, this._ke4) ^ this.f(var5, this._k13);
      var5 = this._k14;
      var10004 = var5 = var10004 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k15);
      var5 = this._k16;
      var10004 = var5 = var10004 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k17);
      var5 = this._k18;
      var5 = var10004 ^ this.f(in1, var5);
      var7 = this._ke5;
      long var10002 = var5 = this.fl(var5, var7);
      in1 = this.flInv(in1, this._ke6) ^ this.f(var5, this._k19);
      var5 = this._k20;
      var10002 = var5 = var10002 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k21);
      var5 = this._k22;
      var10002 = var5 = var10002 ^ this.f(in1, var5);
      in1 ^= this.f(var5, this._k23);
      var5 = this._k24;
      var10002 ^= this.f(in1, var5);
      CamelliaEngine var10003 = this;
      long var9 = in1 ^ this._kw3;
      var5 = var10002 ^ var10003._kw4;
      var10001.wordToBytes(var9, out, outOff);
      var10000.wordToBytes(var5, out, outOff + 8);
      return 16;
   }

   public void init(boolean forEncryption, CipherParameters params) {
      if (params instanceof KeyParameter) {
         this.setKey(forEncryption, ((KeyParameter)params).getKey());
         this.initialised = true;
      } else {
         throw new IllegalArgumentException("only simple KeyParameter expected.");
      }
   }

   public String getAlgorithmName() {
      return "Camellia";
   }

   public int getBlockSize() {
      return 16;
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      if (this.initialised) {
         if (inOff + 16 <= in.length) {
            if (outOff + 16 <= out.length) {
               return this._keyIs128 ? this.processBlock128(in, inOff, out, outOff) : this.processBlock192or256(in, inOff, out, outOff);
            } else {
               throw new DataLengthException("output buffer too short");
            }
         } else {
            throw new DataLengthException("input buffer too short");
         }
      } else {
         throw new IllegalStateException("Camellia engine not initialised");
      }
   }

   public void reset() {
   }
}
